<?php
    namespace Drupal\ldap_auth;

    class LDAPFlow {
        var $ldapconn;
        var $bind;
        var $anon_bind;
        var $server_name;
        var $service_account_username;
        var $service_account_password;
        var $search_base;
        var $search_filter;
        var $custom_base;

        function __construct( ) {
            $this->server_name = \Drupal::config('ldap_auth.settings')->get('miniorange_ldap_server') ? \Drupal::config('ldap_auth.settings')->get('miniorange_ldap_server') : "";
            $this->service_account_username = \Drupal::config('ldap_auth.settings')->get('miniorange_ldap_server_account_username') ? \Drupal::config('ldap_auth.settings')->get('miniorange_ldap_server_account_username') : "";
            $this->service_account_password = \Drupal::config('ldap_auth.settings')->get('miniorange_ldap_server_account_password') ? \Drupal::config('ldap_auth.settings')->get('miniorange_ldap_server_account_password') : "";

        }

        public function getConnection() {
            $this->server_name = $this->getServerName();

            $this->ldapconn = ldap_connect($this->server_name);
            if ( version_compare(PHP_VERSION, '5.3.0') >= 0 ) {
                ldap_set_option($this->ldapconn, LDAP_OPT_NETWORK_TIMEOUT, 5);
            }
            ldap_set_option($this->ldapconn, LDAP_OPT_PROTOCOL_VERSION, 3);
            ldap_set_option($this->ldapconn, LDAP_OPT_REFERRALS, 0);
            $this->anon_bind = $this->getAnonBind();
            if($this->anon_bind)
              $this->setLdapconn($this->ldapconn);
            return $this->ldapconn;
        }

        /**
         * @param mixed $ldapconn
         */
        public function setLdapconn($ldapconn)
        {
          $this->ldapconn = $ldapconn;
        }

        /**
         * @return mixed
         */
        public function getLdapconn()
        {
          return $this->ldapconn;
        }

        /**
         * @return mixed
         */
        public function getServerName()
        {
          return $this->server_name;
        }

        /**
         * @param mixed $server_name
         */
        public function setServerName($server_name)
        {
          \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_server', $server_name)->save();
          $this->server_name = $server_name;
        }

        /**
         * @return mixed
         */
        public function getServiceAccountUsername()
        {
          return $this->service_account_username;
        }

        /**
         * @param mixed $service_account_username
         */
        public function setServiceAccountUsername($service_account_username)
        {
          \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_server_account_username', $service_account_username)->save();
          $this->service_account_username = $service_account_username;
        }

        /**
         * @return array|mixed|string|null
         */
        public function getServiceAccountPassword()
        {
          return $this->service_account_password;
        }

        /**
         * @param mixed $service_account_password
         */
        public function setServiceAccountPassword($service_account_password)
        {
          \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_server_account_password', $service_account_password)->save();
          $this->service_account_password = $service_account_password;
        }


        /**
         * @return mixed
         */
        public function getSearchBase()
        {
          return $this->search_base;
        }

        /**
         * @param mixed $search_base
         */
        public function setSearchBase($search_base, $custom_base)
        {
          if(!is_null($custom_base)){
            \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_custom_sb_attribute', $custom_base)->save();
            $this->$custom_base = $custom_base;
          }
          \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_search_base', $search_base)->save();
          $this->search_base = $search_base;
        }

        /**
         * @return mixed
         */
        public function getAnonBind()
        {
          $this->anon_bind = @ldap_bind($this->getLdapconn()); 			// Anonymous binding with LDAP server. Used to ensure that the LDAP Server is reachable
          LDAPLOGGERS::addLogger('L17: Anonymous LDAP Bind: ',$this->anon_bind);
          return $this->anon_bind;
        }

        /*
         * Test Connection flow
         */
        public function test_mo_ldap_config($username=null, $password=null){
          if(empty($username))
            $username = $this->service_account_username;
          if(empty($password))
            $password = $this->service_account_password;
          if(empty($username) || empty($password)){
            $return_status = array("Username or Password can not be empty until and unless you do not have any authenticaiton setup and wish to perform anonymous bind to your server. If that is the case, please ignore this message and continue with the setup.","error");
            return $return_status;
          }
          $auth_response = $this->ldap_login($username, $password);

          if(!empty($auth_response)){
            if($auth_response->statusMessage == "SUCCESS"){
              $return_status = array("Enable <b>LDAP Login</b> at the top and then Logout from your Drupal site and login again with your LDAP credentials.","Success");
              return $return_status;
            }
            else if($auth_response->statusMessage =="USER_NOT_EXIST"){
              $return_status = array("The user you entered does not exist in the Active Directory. Please check your configurations or contact the administrator","error");
              return $return_status;
            }
            else if($auth_response->statusMessage =="Test_Connection_was_successful"){
              $return_status = array("Your test connection was successful","Success");
              return $return_status;
            }
            else{
              $return_status = array("Invalid Password. Please check your password and try again.","error");
              return $return_status;
            }
          }
        }

        /*
         * LDAP Login flow
         */
      /**
       * Login function
       */
      public function ldap_login($username, $password){
        if(empty($username)){
          \Drupal::messenger()->addError(t("Username can not be empty"));
          return;
        }
        if(empty($password)){
          \Drupal::messenger()->addError(t("The Password can not be empty"));
          return;
        }
        $authStatus = null;
        $auth_response = new Mo_Ldap_Auth_Response();
        $auth_response->userDn = '';
        $ldapconn = $this->getConnection();

        if ($ldapconn) {
          $search_filter = \Drupal::config('ldap_auth.settings')->get('miniorange_ldap_username_attribute');
          $value_filter = '(&(objectClass=*)(' . $search_filter . '=?))';
          $search_bases = \Drupal::config('ldap_auth.settings')->get('miniorange_ldap_search_base');
          if($search_bases == 'custom_base')
            $search_bases = \Drupal::config('ldap_auth.settings')->get('miniorange_ldap_custom_sb_attribute');
          $ldap_bind_dn = $this->service_account_username;
          $ldap_bind_password = $this->service_account_password;
          $filter = str_replace('?', $username, $value_filter);
          $user_search_result = null;
          $entry = null;
          $info = null;
          $fname_attribute = 'mail';
          $lname_attribute = 'dn';
          $email_attribute = 'cn';
          $phone_attribute = 'samaccountname';
          $attr = array($fname_attribute, $lname_attribute, $email_attribute, $phone_attribute);
          $bind = @ldap_bind($ldapconn, $ldap_bind_dn, $ldap_bind_password);
          $err = ldap_error($ldapconn);
          if(strtolower($err) != 'success'){
            $auth_response->status = false;
            $auth_response->statusMessage = 'LDAP_NOT_RESPONDING';
            return $auth_response;
          }
          else if (isset($_COOKIE['Drupal_visitor_mo_ldap_test']) && ($_COOKIE['Drupal_visitor_mo_ldap_test'] == true)){
            $auth_response->status = true;
            $auth_response->statusMessage = 'Test_Connection_was_successful';
            return $auth_response;
          }

          $s1 = ldap_search($ldapconn, $search_bases, $filter);
          if($s1){
            $user_search_result = ldap_search($ldapconn, $search_bases, $filter, $attr);
          }
          else{
            $auth_response->status = false;
            $auth_response->statusMessage = 'USER_NOT_EXIST';
            return $auth_response;
          }
          $info = ldap_first_entry($ldapconn, $user_search_result);
          $entry = ldap_get_entries($ldapconn, $user_search_result);
          if($info){
            $userDn = ldap_get_dn($ldapconn, $info);
          }
          else{
            $auth_response->status = false;
            $auth_response->statusMessage = 'USER_NOT_EXIST';
            return $auth_response;
          }

          $authentication_response = self::authenticate($userDn, $password);
          if($authentication_response->statusMessage == 'SUCCESS'){
            $attributes_array = array();
            $profile_attributes = array();
            $authentication_response->attributeList = $attributes_array;
          }
          return $authentication_response;
        } else{
          $auth_response->status = false;
          $auth_response->statusMessage = 'ERROR';
          return $auth_response;
        }
      }

      /**
       * Authenticate LDAP Credentials
       */
      function authenticate($userDn, $password) {
        $this->ldapconn = ldap_connect($this->server_name);
        if ( version_compare(PHP_VERSION, '5.3.0') >= 0 ) {
          ldap_set_option(null, LDAP_OPT_NETWORK_TIMEOUT, 5);
        }
        ldap_set_option($this->ldapconn, LDAP_OPT_PROTOCOL_VERSION, 3);
        ldap_set_option($this->ldapconn, LDAP_OPT_REFERRALS, 0);
        // binding to ldap server

        $this->bind = @ldap_bind($this->ldapconn, $userDn, $password);
        // verify binding
        $search_filter =\Drupal::config('ldap_auth.settings')->get('miniorange_ldap_username_attribute');
        $value_filter = '(&(objectClass=*)(' . $search_filter . '=?))';
        $filter = str_replace('?', $userDn, $value_filter);
        if ($this->bind) {
          $search_result = ldap_search($this->ldapconn, $userDn,$filter);
          $auth_response = new Mo_Ldap_Auth_Response();
          $auth_response->status = true;
          $auth_response->statusMessage = 'SUCCESS';
          $auth_response->userDn = $userDn;
          return $auth_response;
        }
        $auth_response = new Mo_Ldap_Auth_Response();
        $auth_response->status = false;
        $auth_response->statusMessage = 'ERROR';
        $auth_response->userDn = $userDn;
        return $auth_response;
      }
      /**
       * Returns all search Bases from AD
       */
      public function getSearchBases(){
        $ldapconn = $this->getConnection();
        $ldap_bind_dn = $this->getServiceAccountUsername();
        $ldap_bind_password = $this->getServiceAccountPassword();
        $bind = @ldap_bind($ldapconn, $ldap_bind_dn, $ldap_bind_password);
        $search_base_list = array();
        if($bind){
          $result = ldap_read($ldapconn, '', '(objectclass=*)', array('namingContexts'));
          $data = ldap_get_entries($ldapconn, $result);
          $count = $data[0]['namingcontexts']['count'];
          for ($i = 0; $i < $count; $i++) {
            if ($i == 0) {
              $base_dn = $data[0]['namingcontexts'][$i];
            }
            $valuetext = $data[0]['namingcontexts'][$i];

            array_push($search_base_list, $data[0]['namingcontexts'][$i]);
          }

          $filter = "(|(objectclass=organizationalUnit)(&(objectClass=top)(cn=users)))";
          $search_attr = array("dn", "ou");
          @$ldapsearch = ldap_search($ldapconn, $base_dn, $filter, $search_attr);
          @$info = ldap_get_entries($ldapconn, $ldapsearch);

          for ($i = 0; $i < $info["count"]; $i++) {
            $textvalue = $info[$i]["dn"];
            array_push($search_base_list, $info[$i]["dn"]);
          }
        }
        return $search_base_list;
      }
    }
